@@ -425,10 +425,15 @@ return(va)
 ltmkiener3 <- function(p, m = 0, g = 1, k = 3.2, d = 0, 
                      lower.tail = TRUE, log.p = FALSE) {
 	p  <- if(log.p) {exp(p)} else {p}
-	p  <- if(lower.tail) {p} else {1-p}
-	ltm <- m + g * k / p * (
-	          -pbeta(p, 1+d-1/k, 1-d+1/k) * beta(1+d-1/k, 1-d+1/k) 
-	          +pbeta(p, 1+d+1/k, 1-d-1/k) * beta(1+d+1/k, 1-d-1/k))
+	ltm <- if (lower.tail) {
+		m+g*k/p*(
+			-pbeta(p,1+d-1/k, 1-d+1/k)*beta(1+d-1/k, 1-d+1/k)
+			+pbeta(p,1+d+1/k, 1-d-1/k)*beta(1+d+1/k, 1-d-1/k))	
+	} else {
+		m+g*k/p*(
+			-pbeta(p,1-d+1/k, 1+d-1/k)*beta(1-d+1/k, 1+d-1/k)
+			+pbeta(p,1-d-1/k, 1+d+1/k)*beta(1-d-1/k, 1+d+1/k))
+	}
 return(ltm)
 } 
 
@@ -437,11 +442,15 @@ return(ltm)
 rtmkiener3 <- function(p, m = 0, g = 1, k = 3.2, d = 0, 
                        lower.tail = TRUE, log.p = FALSE) {
 	p   <- if(log.p) {exp(p)} else {p}
-	p   <- if(lower.tail) {p} else {1-p}  
-	# rtm : 1-p et 1-pbeta()
-	rtm <- m + g * k / (1-p) * ( 
-	        (-1+pbeta(p, 1+d-1/k, 1-d+1/k)) * beta(1+d-1/k, 1-d+1/k) 
-	       +( 1-pbeta(p, 1+d+1/k, 1-d-1/k)) * beta(1+d+1/k, 1-d-1/k))
+	rtm <- if (!lower.tail) {
+		m+g*k/(1-p)*(
+			-pbeta(1-p, 1+d-1/k, 1-d+1/k)*beta(1+d-1/k, 1-d+1/k)
+			+pbeta(1-p, 1+d+1/k, 1-d-1/k)*beta(1+d+1/k, 1-d-1/k))	
+	} else {
+		m+g*k/(1-p)*(
+			-pbeta(1-p, 1-d+1/k, 1+d-1/k)*beta(1-d+1/k, 1+d-1/k)
+			+pbeta(1-p, 1-d-1/k, 1+d+1/k)*beta(1-d-1/k, 1+d+1/k))
+	}
 return(rtm)
 }
 
@@ -449,13 +458,13 @@ return(rtm)
 #' @rdname kiener3
 dtmqkiener3 <- function(p, m = 0, g = 1, k = 3.2, d = 0, 
                         lower.tail = TRUE, log.p = FALSE) {
-	p   <- if(log.p) {exp(p)} else {p}
-	p   <- if(lower.tail) {p} else {1-p}
-	dtmq  <- p
+	dtmq <- p
 	for (i in seq_along(p)) {
 		dtmq[i] <- ifelse(p[i] <= 0.5, 
-					  ltmkiener3(p[i], m, g, k, d) - qkiener3(p[i], m, g, k, d),
-					  rtmkiener3(p[i], m, g, k, d) - qkiener3(p[i], m, g, k, d))
+			ltmkiener3(p[i], m, g, k, d, lower.tail, log.p) 
+			- qkiener3(p[i], m, g, k, d, lower.tail, log.p),
+			rtmkiener3(p[i], m, g, k, d, lower.tail, log.p) 
+			- qkiener3(p[i], m, g, k, d, lower.tail, log.p)) 
 	}
 return(dtmq)
 }
